@@ -251,7 +251,6 @@ create_plug (BonoboControl *control)
 	GtkWidget *plug;
 	
 	plug = bonobo_plug_new (0);
-	gtk_widget_show (plug);
 
 	g_object_ref (G_OBJECT (plug));
 	gtk_object_sink (GTK_OBJECT (plug));
@@ -265,6 +264,61 @@ create_plug (BonoboControl *control)
 	g_object_unref (G_OBJECT (plug));
 }
 
+#ifdef HAVE_GTK_MULTIHEAD
+static int
+parse_cookie (const CORBA_char *cookie)
+{
+	GString    *ident = NULL;
+	GString    *value = NULL;
+	const char *p;
+	char       *screen = NULL;
+	int         retval = -1;
+
+	for (p = cookie; *p && !screen; p++) {
+		switch (*p) {
+		case ',':
+			if (!ident || !value)
+				goto parse_failed;
+
+			if (strcmp (ident->str, "screen")) {
+				g_string_free (ident, TRUE); ident = NULL;
+				g_string_free (value, TRUE); value = NULL;
+				break;
+			}
+
+			screen = value->str;
+			break;
+		case '=':
+			if (!ident || value)
+				goto parse_failed;
+			value = g_string_new ("");
+			break;
+		default:
+			if (!ident)
+				ident = g_string_new ("");
+			
+			if (value)
+				g_string_append_c (value, *p);
+			else
+				g_string_append_c (ident, *p);
+			break;
+		}
+	}
+
+	if (ident && value && !strcmp (ident->str, "screen"))
+		screen = value->str;
+
+	if (screen)
+		retval = atoi (screen);
+
+parse_failed:
+	if (ident) g_string_free (ident, TRUE);
+	if (value) g_string_free (value, TRUE);
+
+	return retval;
+}
+#endif /* HAVE_GTK_MULTIHEAD */
+
 static CORBA_char *
 impl_Bonobo_Control_getWindowId (PortableServer_Servant servant,
 				 const CORBA_char      *cookie,
@@ -273,12 +327,29 @@ impl_Bonobo_Control_getWindowId (PortableServer_Servant servant,
 	guint32        x11_id;
 	BonoboControl *control = BONOBO_CONTROL (
 		bonobo_object_from_servant (servant));
+#ifdef HAVE_GTK_MULTIHEAD
+	GdkScreen *gdkscreen;
+	int        screen_num;
+#endif
 
 	if (!control->priv->plug)
 		create_plug (control);
 
 	g_assert (control->priv->plug != NULL);
 
+#ifdef HAVE_GTK_MULTIHEAD
+	screen_num = parse_cookie (cookie);
+	if (screen_num != -1)
+		gdkscreen = gdk_display_get_screen (
+				gdk_display_get_default (), screen_num);
+	else
+		gdkscreen = gdk_screen_get_default ();
+
+	gtk_window_set_screen (GTK_WINDOW (control->priv->plug), gdkscreen);
+#endif
+
+	gtk_widget_show (control->priv->plug);
+
 	x11_id = gtk_plug_get_id (GTK_PLUG (control->priv->plug));
 		
 	dprintf ("plug id %d\n", x11_id);
@@ -339,6 +410,14 @@ impl_Bonobo_Control_setSize (PortableServer_Servant  servant,
 			     const CORBA_short       height,
 			     CORBA_Environment      *ev)
 {
+	GtkAllocation  size;
+	BonoboControl *control = BONOBO_CONTROL (
+		bonobo_object_from_servant (servant));
+	
+	size.x = size.y = 0;
+	size.width = width;
+	size.height = height;
+
 	/*
 	 * Nothing.
 	 *
@@ -346,6 +425,10 @@ impl_Bonobo_Control_setSize (PortableServer_Servant  servant,
 	 * is handled by GtkPlug/GtkSocket for us, or GtkFrame in the
 	 * local case.
 	 */
+
+	g_warning ("setSize untested");
+
+	gtk_widget_size_allocate (GTK_WIDGET (control->priv->plug), &size);
 }
 
 static Bonobo_Gtk_Requisition
@@ -1062,6 +1145,12 @@ window_set_transient_for_gdk (GtkWindow *window,
 			GTK_WIDGET (window)->window, parent);
 	}
 
+	/*
+	 * FIXME: we need to be able to get the screen from
+	 * the remote toplevel window for multi-head to work
+	 * could use an ambient property on the ControlFrame
+	 */
+
 	g_signal_connect (
 		window, "realize",
 		G_CALLBACK (window_transient_realize_gdk_cb), NULL);
@@ -1293,6 +1382,12 @@ bonobo_control_do_popup_full (BonoboControl       *control,
 
 	g_free (path);
 
+#ifdef HAVE_GTK_MULTIHEAD
+	gtk_menu_set_screen (
+		GTK_MENU (menu),
+		gtk_window_get_screen (GTK_WINDOW (control->priv->plug)));
+#endif /* HAVE_GTK_MULTIHEAD */
+
 	gtk_widget_show (menu);
 
 	gtk_menu_popup (GTK_MENU (menu),
