@@ -154,24 +154,26 @@ bonobo_ui_node_add_after (BonoboUINode *before,
 
 /**
  * bonobo_ui_node_insert_before:
- * @sibling: the node to insert
- * @prev_sibling: the placeholder for insertion
+ * @after: the placeholder for insertion
+ * @new_before: the node to insert
  * 
  * Insert a @sibling before @prev_sibling in a node list
  **/
 void
-bonobo_ui_node_insert_before (BonoboUINode *new_before,
-                              BonoboUINode *after)
+bonobo_ui_node_insert_before (BonoboUINode *after,
+			      BonoboUINode *new_before)
 {
+	bonobo_ui_node_unlink (new_before);
+
 	new_before->prev = after->prev;
-	if (!after->prev) {
+
+	if (after->prev) {
+		after->prev->next = new_before;
+	} else {
 		if (after->parent)
 			after->parent->children = new_before;
 	}
 	new_before->next = after;
-	if (new_before->prev)
-		new_before->prev->next = new_before;
-
 	after->prev = new_before;
 
 	new_before->parent = after->parent;
@@ -186,6 +188,9 @@ bonobo_ui_node_insert_before (BonoboUINode *new_before,
 void
 bonobo_ui_node_unlink (BonoboUINode *node)
 {
+	if (!node)
+		return;
+
 	if (!node->prev) {
 		if (node->parent)
 			node->parent->children = node->next;
@@ -343,28 +348,34 @@ get_attr (BonoboUINode *node, GQuark  id, BonoboUIAttr **opt_space)
 	return NULL;
 }
 
-void
-bonobo_ui_node_set_attr_by_id (BonoboUINode *node,
-			       GQuark        id,
-			       const char   *value)
+static inline gboolean
+do_set_attr (BonoboUINode *node,
+	     GQuark        id,
+	     const char   *value)
 {
+	gboolean different = TRUE;
 	BonoboUIAttr *a, *space;
 
-	g_return_if_fail (node != NULL);
+	g_return_val_if_fail (node != NULL, FALSE);
 
 	a = get_attr (node, id, &space);
 
 	if (a) {
-		xmlFree (a->value);
-		a->value = NULL;
+		different = !value || strcmp (a->value, value);
 
-		if (!value) /* Unset the attribute */
-			a->id = 0;
-		else
-			a->value = xmlStrdup (value);
+		if (different) {
+			xmlFree (a->value);
+			a->value = NULL;
+			
+			if (!value) /* Unset the attribute */
+				a->id = 0;
+			else {
+				a->value = xmlStrdup (value);
+			}
+		}
 	} else {
 		if (!value)
-			return;
+			return FALSE;
 
 		if (space) {
 			space->id = id;
@@ -378,6 +389,25 @@ bonobo_ui_node_set_attr_by_id (BonoboUINode *node,
 			g_array_append_val (node->attrs, na);
 		}
 	}
+
+	return different;
+}
+
+void
+bonobo_ui_node_set_attr_by_id (BonoboUINode *node,
+			       GQuark        id,
+			       const char   *value)
+{
+	do_set_attr (node, id, value);
+}
+
+
+gboolean
+bonobo_ui_node_try_set_attr (BonoboUINode *node,
+			     GQuark        prop,
+			     const char   *value)
+{
+	return do_set_attr (node, prop, value);
 }
 
 /**
@@ -607,17 +637,25 @@ bonobo_ui_node_transparent (BonoboUINode *node)
 {
 	gboolean ret = FALSE;
 	static GQuark  name_id = 0;
+	static GQuark  separator_id;
 
 	g_return_val_if_fail (node != NULL, TRUE);
 
-	if (!name_id)
+	if (!name_id) {
 		name_id = g_quark_from_static_string ("name");
+		/* FIXME: ugly to have specific widgets in here */
+		separator_id = g_quark_from_static_string ("separator");
+	}
 
 	if (node->content)
 		ret = FALSE;
 
 	else if (node->attrs->len == 0)
-		ret = TRUE;
+
+		if (node->name_id == separator_id)
+			ret = FALSE;
+		else
+			ret = TRUE;
 
 	else if (node->attrs->len == 1 && attr (node, 0).id == name_id)
 		ret = TRUE;
@@ -775,13 +813,6 @@ uiCharacters (ParseState *ps, const xmlChar *chars, int len)
 	g_string_append_len (ps->content, chars, len);
 }
 
-static xmlEntityPtr
-uiGetEntity (ParseState *ps, const xmlChar *name)
-{
-	/* FIXME: do we need this ? */
-	return xmlGetPredefinedEntity (name);
-}
-
 static void
 uiWarning (ParseState *ps, const char *msg, ...)
 {
@@ -818,7 +849,7 @@ static xmlSAXHandler bonoboSAXParser = {
 	NULL, /* hasInternalSubset */
 	NULL, /* hasExternalSubset */
 	NULL, /* resolveEntity */
-	(getEntitySAXFunc) uiGetEntity, /* getEntity */
+	NULL, /* getEntity */
 	NULL, /* entityDecl */
 	NULL, /* notationDecl */
 	NULL, /* attributeDecl */
@@ -839,6 +870,46 @@ static xmlSAXHandler bonoboSAXParser = {
 	(fatalErrorSAXFunc) uiFatalError, /* fatalError */
 };
 
+static BonoboUINode *
+do_parse (xmlParserCtxt *ctxt)
+{
+	int ret = 0;
+	ParseState *ps;
+	xmlSAXHandlerPtr oldsax;
+
+	if (!ctxt)
+		return NULL;
+
+	ps = parse_state_new ();
+    
+	oldsax = ctxt->sax;
+	ctxt->sax = &bonoboSAXParser;
+	ctxt->userData = ps;
+	/* Magic to make entities work as expected */
+	ctxt->replaceEntities = TRUE;
+
+	xmlParseDocument (ctxt);
+    
+	if (ctxt->wellFormed)
+		ret = 0;
+	else {
+		if (ctxt->errNo != 0)
+			ret = ctxt->errNo;
+		else
+			ret = -1;
+	}
+	ctxt->sax = oldsax;
+	xmlFreeParserCtxt (ctxt);
+
+	if (ret < 0) {
+		g_warning ("XML not well formed!");
+		parse_state_free (ps, TRUE);
+		return NULL;
+	}
+
+	return parse_state_free (ps, FALSE);
+}
+
 /**
  * bonobo_ui_node_from_string:
  * @xml: the xml string
@@ -850,24 +921,15 @@ static xmlSAXHandler bonoboSAXParser = {
 BonoboUINode*
 bonobo_ui_node_from_string (const char *xml)
 {
-	ParseState *ps;
-	GQuark      len;
+	guint len;
 
 	g_return_val_if_fail (xml != NULL, NULL);
 
 	len = strlen (xml);
 	if (len < 3)
 		return NULL;
-	
-	ps = parse_state_new ();
 
-	if (xmlSAXUserParseMemory (&bonoboSAXParser, ps, (char *) xml, len) < 0) {
-		g_warning ("XML not well formed!");
-		parse_state_free (ps, TRUE);
-		return NULL;
-	}
-
-	return parse_state_free (ps, FALSE);
+	return do_parse (xmlCreateMemoryParserCtxt ((char *) xml, len));
 }
 
 /**
@@ -881,19 +943,9 @@ bonobo_ui_node_from_string (const char *xml)
 BonoboUINode*
 bonobo_ui_node_from_file (const char *fname)
 {
-	ParseState *ps;
-
 	g_return_val_if_fail (fname != NULL, NULL);
 	
-	ps = parse_state_new ();
-
-	if (xmlSAXUserParseFile (&bonoboSAXParser, ps, fname) < 0) {
-		g_warning ("XML not well formed!");
-		parse_state_free (ps, TRUE);
-		return NULL;
-	}
-
-	return parse_state_free (ps, FALSE);
+	return do_parse (xmlCreateFileParserCtxt (fname));
 }
 
 /*
@@ -1022,16 +1074,15 @@ validate_tree (BonoboUINode *node)
  * @node: the node tree
  * @recurse: whether to dump its children as well
  * 
- * Convert the Node to its XML string representation
- * see also: bonobo_ui_node_free_string
+ * Convert the node to its XML string representation.
  * 
  * Return value: the string representation or NULL on error
+ * Use g_free to free.
  **/
 char *
 bonobo_ui_node_to_string (BonoboUINode *node,
 			  gboolean      recurse)
 {
-	char    *ret;
 	GString *str = g_string_sized_new (64);
 
 	internal_to_string (str, node, recurse);
@@ -1039,8 +1090,51 @@ bonobo_ui_node_to_string (BonoboUINode *node,
 /*	fprintf (stderr, "nodes to string: '%s'", str->str); */
 /*	validate_tree (node); */
 
-	ret = str->str;
-	g_string_free (str, FALSE);
+	return g_string_free (str, FALSE);
+}
 
-	return ret;
+const char *
+bonobo_ui_node_peek_attr (BonoboUINode *node,
+			  const char   *name)
+{
+	GQuark id = g_quark_from_string (name);
+
+	return bonobo_ui_node_get_attr_by_id (node, id);
+}
+
+/**
+ * bonobo_ui_node_get_path_child:
+ * @node: parent node
+ * @name: 'name' of child node.
+ * 
+ *    Finds the child with the right name, based on
+ * the normal path traversal naming rules.
+ * 
+ * Return value: the child node or NULL.
+ **/
+BonoboUINode *
+bonobo_ui_node_get_path_child (BonoboUINode *node,
+			       const char   *name)
+{
+	BonoboUINode *l;
+	GQuark name_as_quark;
+	static GQuark name_string_id = 0;
+
+	name_as_quark = g_quark_try_string (name);
+
+	if (!name_string_id)
+		name_string_id = g_quark_from_static_string ("name");
+
+	for (l = node->children; l; l = l->next) {
+		BonoboUIAttr *a;
+
+		if ((a = get_attr (l, name_string_id, NULL)) && a->value &&
+		    !strcmp (a->value, name))
+			return l;
+
+		if (l->name_id && l->name_id == name_as_quark)
+			return l;
+	}
+
+	return NULL;
 }
