@@ -18,14 +18,15 @@
 #include <bonobo/bonobo-ui-node-private.h>
 #include <bonobo/bonobo-ui-toolbar-control-item.h>
 
-/* To debug plug/socket/control render issues */
-#undef DEBUG_CONTROL
+G_BEGIN_DECLS
 
 /* To dump lots of sequence information */
-#undef DEBUG_UI
+#define noDEBUG_UI
 
-void       bonobo_ui_gconf_leaks_refs     (void);
-void       bonobo_ui_preferences_shutdown (void);
+/* To debug render issues in plug/socket/control */
+#define noDEBUG_CONTROL
+
+int        bonobo_ui_preferences_shutdown (void);
 void       bonobo_ui_image_set_pixbuf     (GtkImage       *image,
 					   GdkPixbuf      *pixbuf);
 GdkPixbuf *bonobo_ui_util_xml_get_pixbuf  (GtkWidget      *widget,
@@ -36,6 +37,10 @@ void       bonobo_ui_toolbar_control_item_set_sensitive
                                           (BonoboUIToolbarControlItem *item,
 					   gboolean                    sensitive);
 
+
+gboolean   bonobo_widget_clobber_focus    (GtkWidget        *widget,
+					   GtkDirectionType  direction);
+
 #ifndef   DEBUG_UI
 
 static inline void dprintf (const char *format, ...) { };
@@ -48,5 +53,7 @@ static inline void dprintf (const char *format, ...) { };
 
 #endif /* DEBUG_UI */
 
+G_END_DECLS
+
 #endif /* _BONOBO_UI_PRIVATE_H_ */
 
