@@ -221,6 +221,8 @@ display_as_control (const char *moniker, CORBA_Environment *ev)
 
 		widget = bonobo_widget_new_control_from_objref (
 			the_control, BONOBO_OBJREF (ui_container));
+
+		bonobo_object_release_unref (the_control, ev);
 	}
 
 	if (BONOBO_EX (ev) || !widget)
@@ -232,8 +234,8 @@ display_as_control (const char *moniker, CORBA_Environment *ev)
 
 	bonobo_window_set_contents (BONOBO_WINDOW (window), widget);
 
-	g_signal_connect (GTK_OBJECT (window), "destroy",
-			    G_CALLBACK (bonobo_main_quit), NULL);
+	g_signal_connect (window, "destroy",
+			  G_CALLBACK (bonobo_main_quit), NULL);
 
 	gtk_widget_show_all (window);
 	bonobo_main ();
@@ -288,8 +290,9 @@ main (int argc, char **argv)
 		exit (1);
 	}
 
-	if (bonobo_ui_init ("test-moniker", "1.0", &argc, argv) == FALSE)
-		g_error ("Cannot init bonobo");
+	gnome_program_init ("test-moniker", VERSION,
+			    LIBBONOBOUI_MODULE,
+			    argc, argv, NULL);
 
 	if (global_mto.ps + global_mto.pr + global_mto.pc > 1) {
 		poptPrintUsage (ctx, stderr, 0);
@@ -351,7 +354,7 @@ main (int argc, char **argv)
 	do_moniker_magic ();
 
 	while (global_mto.async && !async_done)
-		g_main_iteration (TRUE);
+		g_main_context_iteration (NULL, TRUE);
 
 	return bonobo_ui_debug_shutdown ();
 }
